#!/usr/bin/perl -w

# Nom du programme:find_and_mask_ms.pl (modified from mask_ms.pl and extract_perfect_microsat_with_hach.pl)

#INPUT: xxx.fas (file with all sequences in it (adaptor already cut))

# Finds all microsat of a sequence 
# $minx is the minimum number of repeats for all motifs, where X is the lenght of the motif (e.g. $min2 is the minimum mumber of dinucleotide repeats)

# Put sequences into fasta file 
# works with multiple files with multiple sequences in it
# ONE FILE PER SPECIES

#OUTPUT:
#xxx_mask.fas # if $keep_non_ms == 1 writes all sequences infor fasta file, If 0, writes aonly sequences with ms in it in fasta file (ms is masked by lowercase))
#(fasta file with all sequences in it, ms (if present) are masked by lowercase letters)


# xxx.seq (info on seq)
# [0] SEQ_CODE
# [1] NUMBER OF MS
# [2] SEQ_LENGTH
# [3] MS MOTIF
# [4] MS FIRTS POS
# [5] MS LAST POS
# [6] NUMBER OF REP
# [3-6] REPEATED FOR ALL MS



  use warnings;
  use strict;
  use subprogramQDD;


 # my $USAGE = "USAGE : $0 filename_with_pathway folder_for_the_outfiles minimum_number_of_repeats_for_2-3-4-5-6-bp_motifs minimum_number_of_repeats_for_single-bp_motifs E_value_for_checking_vector_contamination pathway_for_BLAST\n\n";

	my $min1 = 0;
	my $min2 = 0;
	my $min3 = 0;
	my $min4 = 0;
	my $min5 = 0;
	my $min6 = 0;
	my $del_prev_mask = 0;
	my $keep_non_ms = 0;
	my $folder = '';
	my $outfolder = '';
	my $file_motif = '';
  my $write_seq_file = '';
  my $write_fas_file = '';
  my $one_file = '';
 if ( scalar@ARGV == 14 )
 {
  $min1 = $ARGV[0];
	$min2 = $ARGV[1];
	$min3 = $ARGV[2];
	$min4 = $ARGV[3];
	$min5 = $ARGV[4];
	$min6 = $ARGV[5];
	$del_prev_mask = $ARGV[6];
	$keep_non_ms = $ARGV[7];
	$folder = $ARGV[8];
	$outfolder = $ARGV[9];
	$file_motif = $ARGV[10];
	$write_seq_file = $ARGV[11];
  $write_fas_file = $ARGV[12];
  $one_file = $ARGV[13];
 }

else
{
	$min1 = 5;
	$min2 = 3;
	$min3 = 3;
	$min4 = 3;
	$min5 = 3;
	$min6 = 3;
# if $del_prev_mask = 0; keeps previous lower case masking, else deletes previous masking
	$del_prev_mask = 1;
# if 1 writes all sequences info fasta file, If 0, writes only sequences with ms in it in fasta file (ms is masked by lowercase))
	$keep_non_ms = 0;
	$folder = 'temp';
	$outfolder = 'temp';
	$file_motif = '\.fas';
	$write_seq_file = 1; # if 1 seq_file is produced
  $write_fas_file = 1; # if 1 masked fasta file is produced
  $one_file = 0; # if one instead of infolder there is a name of the only file, that should be treated
	print "Direct parameteres are used in find_and_mask_ms!!\n";
}

my $time_beg = time;

my @filenames = '';
if ($one_file ==1)
{
  $folder=~ /(.*[\\\/])([^\\^\/]*)/;
  my $file = $2;
  $folder = $1;
 # print "$file, $folder\n";
  push(@filenames, $file); 
}
else
{
 unless ( opendir(FOLDER, $folder) )
 {
   print "Cannot access to folder $folder\n";
   exit;
 }
@filenames = grep ( !/^\.\.?$/, readdir(FOLDER) );
closedir(FOLDER);
}

foreach my $filename (@filenames)
{
if ($filename =~ /$file_motif$/)
{
  if ($write_seq_file ==1)
  {
  my $seq_file = $filename;
  $seq_file = $outfolder.'/'.$seq_file; 
  $seq_file =~ s/\..*/\.seq/;
  open_file_out($seq_file, 'SEQ');
  print SEQ "seq_code;number_of_ms;seq_length;ms_motif;ms_first_pos;ms_last_pos;number_of_rep\n";
  }

  if ($write_fas_file ==1)
  {
	my $fas_file = $filename;
	$fas_file = $outfolder.'/'.$fas_file;
	$fas_file =~ s/\./_mask\./;
	open_file_out($fas_file, 'FAS');
  }
 # unless ($one_file ==1)
 # {
  $filename = $folder.'/'.$filename; 
 # }

	my $m = 0;


  open_file_in($filename, 'DATA');
  while (my $seq = one_seq($filename))
  {
	if ($seq eq '>')
	{
		next;
	}
	$seq =~ /.+\n/;
	my $code = $&;
	$seq =~ s/$code//;
	$seq =~ s/>//;
	$seq =~ s/\s//g;
	chomp $seq;
	$code =~ s/\s.*//;
	chomp $code;

  my %seq1 = ();
  my $seq_length = length $seq;
	my %ms_motif = ();
	my %ms_pos = ();

	if ($del_prev_mask == 1)
	{
		$seq = uc $seq;
	}
#	print "$seq\n";
#¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤
	find_microsat_gb_motif_def_6($seq, \%ms_motif, \%ms_pos, $min1, $min2, $min3, $min4, $min5, $min6);
#¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤
    my @ms = sort {$a <=> $b} keys %ms_pos;
    my $ms_numb = scalar @ms;
if ($write_fas_file == 1)
{
	if (scalar@ms > 0)
	{
		++$m;
		    my $fragment_l = change_ms_into_lower_case($seq, \%ms_pos);
		print FAS ">$code\n$fragment_l\n";
	}
	elsif ($keep_non_ms == 1)
	{
		print FAS ">$code\n$seq\n";
	}
	else
	{
		next;
	}
}

#print info into seq file
if ($write_seq_file == 1)
{
    print SEQ "$code;$ms_numb;$seq_length;";
    foreach my $ms (@ms)
    {
	my $ms_length = $ms_pos{$ms} - $ms +1;
	my $rep_length = length $ms_motif{$ms};
	my $rep_numb = $ms_length/$rep_length;
	print SEQ "$ms_motif{$ms};$ms;$ms_pos{$ms};$rep_numb;";
    }
    print SEQ "\n";   
}

#print_hachage(\%ms_pos);
#print_hachage(\%ms_motif);

# while
  }
  close DATA;
  if ($write_seq_file ==1)
  {
  close SEQ;
  }
    if ($write_fas_file ==1)
  {
  close FAS;
  }

#print "Number of sequences with ms in it: $m\n";
# end if file_motif
}
#end of foreach file
}

my $run_time = time - $time_beg;
#print "The run took $run_time s\n";
exit;


############################################################


sub one_seq
{
#    read the data concernant one sequence

    my ($filename) = @_; 
    my ($res_seq) = ''; 

    my ($sep_entree) = $/;
    $/ = ">";
    $res_seq = <DATA>;
    $/ = $sep_entree;
    return $res_seq;
}


##########################################################
