#!/usr/bin/perl -w


  use warnings;
  use strict;
  use subprogramQDD;
#  use Data::Dumper;
  
  
#INPUT: fasta file
# output: 

# if $direct_par_def = 0 (command line menu) set_file is read and the user is asked if the settings are correct and can change them 
# if $direct_par_def = 1 all parameters are defined in the program; 
# if $direct_par_def = 2 the set_file is read and the user cannot change the parameters (option for running behind GUI)

my $direct_par_def = 0;

my $min1 = 1000000;
my $min2 = 5;
my $min3 = 5;
my $min4 = 5;
my $min5 = 5;
my $min6 = 5;
my $e_tag = '1E-1';
my $word_size = 7; 
my $Evector = '1E-3';
my $set_file = 'set_pipe1.txt'; 
my $set_file_default = 'set_pipe1_default.txt';
my $blast_version = 'blast+';

print_heading();

my %param = (
  syst => '',  
  blast_path => '',
  project_folder => '',
  tag => '',
  adapter => '',
  length_limit => '',
  del_files => ''
);
my %param_list = define_param_number_pipe1(); # $param_list{1}[syst, 'Operating system (win/linux):'], ...
my %param_limits = define_param_limits_pipe1(\%param); #hash of array $param_limits{syst}[type(list, real, folder), error message, ['list of values/lower_upper limits)']

if ($direct_par_def == 1) # define parameteres values in the script
{
  $param{syst} = 'win';  
#  $param{blast_path} = 'c:/BLAST2_2_18/bin/';
  $param{blast_path} = 'c:\Program files\NCBI\blast-2.2.25+\bin\\';
#  $param{blast_version} = 'blast+'; # blast/blast+
  $param{tag} = 0;
  $param{adapter} = 1;
  $param{length_limit} = 80;
  $param{del_files} = 0;
  $param{project_folder} = 'd:/test with space';
}
elsif($direct_par_def == 2) # for GUI
{
  read_set_file_for_GUI(\%param, $set_file, $set_file_default);
}
elsif($direct_par_def == 0)
{
  read_set_file_for_command_line(\%param, $set_file, \%param_list, $set_file_default);
}

check_param_limits(\%param, \%param_limits);


my $outfolder = time;
my $date = localtime $outfolder;

my $tag_file = 'tag.fas';
my $adapter_file = 'adapter.fas';

my $pipe1_folder = $param{project_folder}.'/pipe1_'.$outfolder;
my $log_file = $pipe1_folder.'/pipe1_log_'.$outfolder.'.txt';
make_folder($param{syst}, $pipe1_folder);
make_folder($param{syst}, $outfolder);

my %data1_list = get_file_folder_list($param{project_folder});

$tag_file = copy_tag_and_adapter_file(\%data1_list, $param{project_folder}, $tag_file, $outfolder, $param{tag}, $param{syst});
$adapter_file = copy_tag_and_adapter_file(\%data1_list, $param{project_folder}, $adapter_file, $outfolder, $param{adapter}, $param{syst});


print "Input folder: $param{project_folder}\n";
print "Output pipe1 folder: $pipe1_folder\n\n";

  open_file_out($log_file, 'LOG');
  print LOG "**********************************************************\n";
  print LOG "QDD version2.1(beta) 28 June 2011\n";
  print LOG "Emese Meglecz, Aix-Marseille University, Marseille, France\n";
  print LOG 'emese.meglecz@univ-provence.fr';
  print LOG "\n";
  print LOG "Plesae, read the Documentation_QDD2.pdf\n";
  print LOG "**********************************************************\n\n";
  print LOG "Output folder identifyer: $outfolder\n";
  print LOG "Run started at : $date\n";
  print LOG "Minimum homopolymer length for microsatellite detection: $min1\n";
  print LOG "Minimum repeat number for dibase motifs for microsatellite detection: $min2\n";
  print LOG "Minimum repeat number for tribase motifs for microsatellite detection: $min3\n";
  print LOG "Minimum repeat number for tetrabase motifs for microsatellite detection: $min4\n";
  print LOG "Minimum repeat number for pentabase motifs for microsatellite detection: $min5\n";
  print LOG "Minimum repeat number for hexabase motifs for microsatellite detection: $min6\n";
  print LOG "E-value for tag detection: $e_tag\n";
  print LOG "E-value for adapter detection: $Evector\n";
  print LOG "Word size for tag and adapter detection: $word_size\n";

  print LOG "System (win/linux): $param{syst}\n";
  print LOG "Path to BLAST executables: $param{blast_path}\n";
#  print LOG "BLAST version(blast/blast_plus): $param{blast_version}\n";
  print LOG "Sort by tag (YES=1/NO=0): $param{tag}\n";
  print LOG "Remove adapter (YES=1/NO=0): $param{adapter}\n";
  print LOG "Minimum sequence length: $param{length_limit}\n";
  print LOG "Delete intremediate files (YES=1/NO=0): $param{del_files}\n";
  print LOG "Project folder: $param{project_folder}\n";
  print LOG "Output folder: $pipe1_folder\n";
  print LOG "\n************************************\n\n";


foreach my $file (sort keys %data1_list)
{

if ($file =~ /\.fas/)
{  
  my $time_beg = time;
  my $file_root = $file;
  $file_root =~ s/\.fas//;  
  my $outfolder_spec = '';
  $outfolder_spec = $outfolder.'/'.$file_root.'_'.$outfolder;
  make_folder($param{syst}, $outfolder_spec);
############################################## star prg

print "\n";
print "**********\n$file\n**********\n";
print LOG "**********\n$file\n**********\n";
# Sort sequences by TAG
  my %inp = (); # file_name as key , path to that file as value
  if ($param{tag})
  {
    my $sort_tag = 'perl sort_tag.pl '.$param{syst}.' "'.$file.'" "'.$param{project_folder}.'" "'.$outfolder_spec.'" "'.$param{blast_path}.'" '.$blast_version.' '.$e_tag.' '.$word_size.' "'.$tag_file.'"';
# params:  	$syst	$fas_file(wo path) $fas_folder (path to fasta file)	$outfolder $blast_path $blast_version $Evector	$word_size $tag_file(wo path) 

    print LOG "$sort_tag\n";
    print "Sorting sequences in $file by tags\n";
    system $sort_tag;
    get_sorted_files(\%inp, $outfolder_spec, '\.tag'); # make a hash with input filenames for further steps as keys, path to these files as value
    delete_notag(\%inp); # delete xxx_NOTAG.tag from list
  }
  else
  {
    $inp{$file} = $param{project_folder};
  }

# remove adapter
  if ($param{adapter})
  {
    foreach my $wotag_file (sort keys %inp)
    {
      my $cut_vector = 'perl cut_vector.pl '.$param{syst}.' "'.$wotag_file.'" "'.$inp{$wotag_file}.'" "'.$outfolder_spec.'" "'.$param{blast_path}.'" '.$blast_version.' '.$Evector.' "'.$adapter_file.'" 1';    
   #$syst $fas_file $fas_folder $outfolder $blast_path $blast_version $Evector $adapt_file $noadapt 
      print LOG "$cut_vector\n";
      print "Removing adapters from sequences in $wotag_file\n";
      system $cut_vector;
    }
    %inp = ();
    get_sorted_files(\%inp, $outfolder_spec, '\.wov'); # make a hash with input filenames for further steps as keys, path to these files as value
  }
  

# remove short sequences
  foreach my $clean_file (sort keys %inp)
  {
     my $remove_short = 'perl remove_short_seq.pl '.$param{syst}.' "'.$clean_file.'" "'.$inp{$clean_file}.'" "'.$outfolder_spec.'" '.$param{length_limit};    
     #	$syst $fas_file $fas_folder $outfolder $limit
     print LOG "$remove_short\n";
     print "Removing sequences shorter than $param{length_limit} bp from $clean_file\n";
     system $remove_short;
  }

# find sequences with MS in all files xxxbp.fas in the $outfolder_spec
      my $find_ms = 'perl find_and_mask_ms2.pl '.$min1.' '.$min2.' '.$min3.' '.$min4.' '.$min5.' '.$min6.' 1 0 "'.$outfolder_spec.'" "'.$outfolder_spec.'" bp\.fas 1 1 0';
    #  USAGE: $min1 $min2 $min3 $min4	$min5	$min6	$del_prev_mask $keep_non_ms $folder/$file	$outfolder $file_motif $write_seq_file $write_fas_file $one_file;
     print LOG "$find_ms\n";
     print "Selecting sequences with microsatellites\n";
     system $find_ms;
     

############################################## end prg

  my %motif_list = initialise_motif_list($file);
  my %seq_number =();
  get_seq_number(\%motif_list,  \%seq_number, $outfolder_spec);
  get_seq_number(\%motif_list,  \%seq_number, $param{project_folder});
  my @mot_list = ($file, 'NOTAG\.tag', '\.woa', '\.wov', 'bp.fas', 'bp_mask\.fas');


  print LOG "\n\n";
  foreach my $motif (@mot_list)
  {
    if (exists $seq_number{$motif})
    {
    print LOG "$motif_list{$motif}:\n";
    foreach my $code (sort keys %{$seq_number{$motif}})
    {
      print LOG "\t$code: $seq_number{$motif}{$code}\n";
    }
    }
  }

  move_files($outfolder_spec, $pipe1_folder, $param{syst}, '\.woa');
  move_files($outfolder_spec, $pipe1_folder, $param{syst}, '\.wov');
  move_files($outfolder_spec, $pipe1_folder, $param{syst}, 'bp\.seq');
  move_files($outfolder_spec, $pipe1_folder, $param{syst}, 'length\.tbl');
  move_files($outfolder_spec, $pipe1_folder, $param{syst}, 'NOTAG\.tag');
  rename_files($outfolder_spec, $pipe1_folder, $param{syst}, $param{length_limit}.'bp_mask\.fas', 'pipe2.fas');

  print LOG "\n";
  my $time_end = time;
  my $time_run = $time_end - $time_beg;
  print LOG "\nThe run took $time_run s.\n\n";
  


} # end if file motif
} # end foreach my $file

close LOG;

if ($param{tag})
{
  delete_tag_adapt_files($tag_file, $param{syst});
}
if ($param{adapter})
{
  delete_tag_adapt_files($adapter_file, $param{syst});
}


if ($param{del_files})
{
  delete_folder($outfolder, $param{syst});
}

print "\nThe anlysis is finished\n";

exit;


######################################################

sub define_param_number_pipe1
{
  my %param_list = (
      1=> ['syst', 'Operating system (win/linux):'], 
      2=> ['project_folder', 'Project folder:'], 
      3=> ['tag', 'Sort sequences by tag (YES=1/NO=0):'],
      4=> ['adapter', 'Remove adapters from sequences (YES=1/NO=0):'], 
      5=> ['length_limit', 'Minimum sequence length:' ], 
      6=> ['blast_path', 'Pathway to BLAST executables:'], 
 #     7=> ['blast_version', 'BLAST version (blast/blast+):'],
      7=> ['del_files', 'Delete intermediate files (YES=1/NO=0):']
      );
      
  del_files => '',

  return %param_list;
}


##################################################################

sub define_param_limits_pipe1
{
my ($param) = @_;
#hash of array [type(list, real, folder), error message, ['list of values/lower_upper limits)']
  my %param_limits = (
      'syst'=> ['list', 'Operating system must be win or linux', ['win', 'linux']], 
      'project_folder' => ['folder', "Cannot open the Project folder $$param{project_folder}"], 
      'blast_path'=> ['folder', "Cannot open $$param{blast_path} folder"], 
#      'blast_version'=> ['list', 'blast_version must be blast or blast+', ['blast','blast+']],
      'tag'=> ['list', 'Tag must be 0 or 1', [0,1]],
      'adapter'=> ['list', 'Adapter must be 0 or 1', [0,1]],
      'length_limit'=> ['real', 'Minimum sequence length must be between 1 and 10000', [1,10000]],
      'del_files'=> ['list', 'del_files must be 0 or 1', [0,1]]
      );
}
##################################################################


sub copy_tag_and_adapter_file
{
my ($data1_list, $source_folder, $file, $outfolder, $bool, $syst) = @_;

my $file_new = $outfolder.'_'.$file;

if ($bool)
{
  if (exists $$data1_list{$file})
  {
    copy_one_file($source_folder, $file, $file_new, $syst);
    delete $$data1_list{$file};
  }
  else
  {
    print "The $file file is not in in the $source_folder folder\n";
    exit;
  }
}
elsif (exists $$data1_list{$file})
{
  delete $$data1_list{$file};
}

return $file_new;
}


###################################################

sub get_sorted_files
{
 my ($adapt_inp, $folder, $motif) = @_;
 my %temp = get_file_folder_list($folder);
 foreach my $file (keys %temp)
 {
  if ($file =~ /$motif/)
  {
    $$adapt_inp{$file} = $folder;
  }
 }
 %temp = ();
}

###################################################

sub delete_notag
{
my ($inp) = @_;

foreach my $file (keys %$inp)
{
  if ($file =~ /_NOTAG\./)
  {
    delete $$inp{$file};
  }
}
}

###################################################

sub get_seq_number
{
 my ($motif_list,  $seq_number, $folder) = @_;
 
unless ( opendir(FOLDER, "$folder") )
{
      print "Cannot access to folder $folder\n";
      exit;
}
my @files = grep ( !/^\.\.?$/, readdir(FOLDER) );
closedir(FOLDER);

foreach my $file (@files)
{
  foreach my $motif (keys %$motif_list)
  {
    if ($file =~ /$motif/)
    {
      open_file_in($folder.'/'.$file, 'IN');
      my $i = 0;
      while (my $line = <IN>)
      {
        if ($line =~ />/)
        {
          ++$i;
        }
      }
      $$seq_number{$motif}{$file} = $i;
      close IN;
    }
  }
}

 
}

###############################################################
sub initialise_motif_list
{
  my ($file) = @_;
 my %motif_list = ();

  $motif_list{$file} = "The number of sequences in $file";
  $motif_list{'\.woa'}= "The number of sequences without adapter at the begining (not used for microsat research)";
  $motif_list{'\.wov'} = "The number of sequences with adapter at the begining";
  $motif_list{'NOTAG\.tag'} = "The number of sequences without TAG (not used for microsat research)";
  $motif_list{'bp_mask\.fas'} = "The number of sequences with at least one microsatellite";
  $motif_list{'bp.fas'} = "The number of sequences of at least $param{length_limit} bp (without tag and adapter)";

  return %motif_list;
}

#################################################################

sub delete_tag_adapt_files
{
my ($file, $syst) = @_;
if ($syst eq 'win')
{
  system 'del '.$file.'*';
}
else
{
  system 'rm '.$file.'*';
}
}
