#!/usr/bin/perl -w


  use warnings;
  use strict;
#  use subprogramEM;
  use subprogramQDD;
#  use Data::Dumper;
  
  
#INPUT: fasta file with clean (no adapter, no tag) microsatellite containing sequences.
# output: 
# file_root_unique.fas => the only BLASt hit is autohit; All >=4 repeat microsatellites and >=8 homoplymers are masked
# file_root_nohit_css.fas => no hit to itself; All >=4 repeat microsatellites and >=8 homoplymers are masked
# file_root_multihit_css.fas => more than one hit between a pair of sequences; All >=4 repeat microsatellites and >=8 homoplymers are masked
# file_root_gr.fas => sequences (including consensuses) that had BLAST hit to other sequences, with bellow limit identity of the overlaping region. Regions covered by BLAST hits are masked 
# file_root_consensus.fas => all unique (no hit to grouped seqs) consensus sequences, if polymorphism of ms motif_first_last pos of MS (in the aligned cons seq) is given; All >=4 repeat microsatellites are masked, but not homopolymers;
# file_root_cons_subs.fas => Consensus + aligned sequences to make a consensus ; All >=4 repeat microsatellites are masked, but not homopolymers; grouped consensus sequences are NOT removed
# file_root_cons_log.txt


# if $direct_par_def = 0 (command line menu) set_file is read and the user is asked if the settings are correct and can change them 
# if $direct_par_def = 1 all parameters are defined in the program; 
# if $direct_par_def = 2 the set_file is read and the user cannot change the parameters (option for running behind GUI)

my $direct_par_def = 0;
my $set_file = 'set_pipe2.txt';
my $set_file_default = 'set_pipe2_default.txt';
my $mask_mono = 8;
my $mask_di = 4;
my $mask_tri = 4;
my $mask_tetra = 4;
my $mask_penta = 4;
my $mask_hexa = 4;
my $e = 1e-40;
my $blast_version = 'blast+';

print_heading();

my %param = (
  syst => '',  
  blast_path => '',
#  blast_version => '',
  make_cons => '', # if 0 make only a file with unique autohit sequences
  ident_limit => '',
  clust_path => '',
  prop_maj => '',
  project_folder => '',
  del_files => ''
);
my %param_list = define_param_number_pipe2(); # $param_list{1}[syst, 'Operating system (win/linux):'], ...
my %param_limits = define_param_limits_pipe2(\%param); #hash of array $param_limits{syst}[type(list, real, folder), error message, ['list of values/lower_upper limits)']

if ($direct_par_def == 1) # define parameteres values in the script
{
  $param{syst} = 'win';  
  $param{blast_path} = 'c:/BLAST2_2_18/bin/';
# $param{blast_path} = 'c:\Program files\NCBI\blast-2.2.25+\bin\\';
#  $param{blast_version} = 'blast'; # blast/blast+
  $param{make_cons} = 1; # if 0 make only a file with unique autohit sequences
  $param{ident_limit} = 95;
  $param{clust_path} = 'c:/CLUSTALW/clustalw2/';
  $param{prop_maj} = 0.66;
  $param{project_folder} = 'd:/test with space';
  $param{del_files} = 1;
}
elsif($direct_par_def == 2) # for GUI
{
  read_set_file_for_GUI(\%param, $set_file, $set_file_default);
}
elsif($direct_par_def == 0)
{
  read_set_file_for_command_line(\%param, $set_file, \%param_list, $set_file_default);
}

check_param_limits(\%param, \%param_limits);

#determine the last created folder within $project_folder, that contains motif ('^pipe1');
my $pipe1_folder = determine_last_folder($param{project_folder}, '^pipe1');
my %pipe1_list = get_file_folder_list($pipe1_folder);
my $outfolder = time;
my $date = localtime $outfolder;
make_folder($param{syst}, $outfolder);
my $pipe2_folder = $param{project_folder}.'/pipe2_'.$outfolder;
make_folder($param{syst}, $pipe2_folder);
my $log_file = $pipe2_folder.'/pipe2_log_'.$outfolder.'.txt';

  open_file_out($log_file, 'LOG');
  print LOG "**********************************************************\n";
  print LOG "QDD version2.1(beta) 28 June 2011\n";
  print LOG "Emese Meglecz, Aix-Marseille University, Marseille, France\n";
  print LOG 'emese.meglecz@univ-provence.fr';
  print LOG "\n";
  print LOG "Plesae, read the Documentation_QDD2.pdf\n";
  print LOG "**********************************************************\n\n";
  print LOG "Output folder identifyer: $outfolder\n";
  print LOG "Run started at : $date\n";
  print LOG "Minimum homopolymer mength for soft masking: $mask_mono\n";
  print LOG "Minimum repeat number for dibase motifs for sof masking: $mask_di\n";
  print LOG "Minimum repeat number for tribase motifs for sof masking: $mask_tri\n";
  print LOG "Minimum repeat number for tetrabase motifs for sof masking: $mask_tetra\n";
  print LOG "Minimum repeat number for pentabase motifs for sof masking: $mask_penta\n";
  print LOG "Minimum repeat number for hexabase motifs for sof masking: $mask_hexa\n";
  print LOG "E-value for all-against-all BLAST: $e\n";

  print LOG "System (win/linux): $param{syst}\n";
  print LOG "Path to BLAST executables: $param{blast_path}\n";
#  print LOG "BLAST version(blast/blast_plus): $param{blast_version}\n";
  print LOG "Make consensus sequences(0/1): $param{make_cons}\n";
  print LOG "Minimum identity between two sequences to make a consensus:$param{ident_limit}\n";
  print LOG "Path to CLustal2 executables: $param{clust_path}\n";
  print LOG "Proportion of sequences that must have the same base on the aligned site to accept it as a consensus: $param{prop_maj}\n";
  print LOG "Project folder: $param{project_folder}\n";
  print LOG "Delete intremediate files: $param{del_files}\n";
  print LOG "Pipe1 folder: $pipe1_folder\n";
  print LOG "Pipe2 folder: $pipe2_folder\n";
  print LOG "\n************************************\n";


print "Input pipe1 folder: $pipe1_folder\n";
print "Output pipe2 folder: $pipe2_folder\n";

foreach my $file (sort keys %pipe1_list)
{

if ($file =~ /_pipe2\.fas/)
{  
  my $time_beg = time;

  my $file_root = $file;
  $file_root =~ s/\.fas//;
  
  my $outfolder_spec = '';
  my $temp_folder = '';
  $outfolder_spec = $outfolder.'/'.$file_root.'_'.$outfolder;
  $temp_folder = $outfolder_spec.'/temp';

  make_folder($param{syst}, $outfolder_spec);
  make_folder($param{syst}, $temp_folder);

  my $unique_file = $outfolder_spec.'/'.$file_root.'_unique.fas';
  my $nohit_css_file = $outfolder_spec.'/'.$file_root.'_nohit_css.fas';
  my $multihit_css_file = $outfolder_spec.'/'.$file_root.'_multihit_css.fas';
  my $gr_file = $outfolder_spec.'/'.$file_root.'_gr.fas';
  my $consensus_file = $outfolder_spec.'/'.$file_root.'_consensus.fas';
  my $cons_subs_file = $outfolder_spec.'/'.$file_root.'_cons_subs.fas';
  my $pipe3_input_file = $pipe2_folder.'/'.$file_root.'.fas';
  $pipe3_input_file =~ s/pipe2\.fas/pipe3\.fas/;

  my %seq_count = ();
  
  $file = $pipe1_folder.'/'.$file;
  print "\n******************\n$file\n******************\n";
  print LOG "\n******************\n$file\n******************\n\n";
    # mask MS and homopolymers in the input fasta file use low limit values to avoid false BLAST hits  and eliminate CSS sequences (no autohit)
  my $find_ms1 = 'perl find_and_mask_ms2.pl '.$mask_mono.' '.$mask_di.' '.$mask_tri.' '.$mask_tetra.' '.$mask_penta.' '.$mask_hexa.' 1 1 "'.$file.'" "'.$temp_folder.'" pipe2\.fas 1 1 1';
    #  USAGE: $min1 $min2 $min3 $min4	$min5	$min6	$del_prev_mask $keep_non_ms $folder/$file	$outfolder $file_motif $write_seq_file $write_fas_file $one_file;
	print LOG "\n$find_ms1\n";
  print "Soft masking sequences\n";
  system $find_ms1;
 
  $file = $temp_folder.'/'.$file_root.'_mask.fas';
  my $db_file = $outfolder.'_'.$file_root.'_mask.fas';
  copy_one_file($temp_folder, $file_root.'_mask.fas', $db_file, $param{syst});

  my $blast_out = $temp_folder.'/'.$file_root.'_mask_blast.tbl';
  print "All-against-all BLAST\n";
  call_blast_query_to_db_soft_mask($file, $db_file, $blast_out, $e, $param{blast_path}, $blast_version, $param{syst});
#  call_blast_tbl_soft_filter($file, $blast_out, $e, $param{blast_path}, $param{blast_version});
 # delete_blast_temp_files_win_linux($outfolder_spec, $param{syst});
 delete_tag_adapt_files($db_file, $param{syst});

  if ($param{make_cons} == 0)
  {
    $unique_file = $pipe3_input_file;
  }
  print "Select unique sequences\n";
  my %seq = read_hach_from_fasta_code_wo_space($file);
  $seq_count{'Total number of input sequences:'} = scalar keys%seq;
  make_unique_only(\%seq, $unique_file, $nohit_css_file, $blast_out, \%seq_count);

 if ($param{make_cons} == 1)
 {
  my %blast = (); # hash of anonymous hashes $blast{code1}{code2}=>[blast ident, qfirts, qlast, sfirst, slast]
      # does not contain autohits; only one hit is recorded in case of reciproc hits; 
      # several hits between the two same sequences is marked as css
  my %css =(); # seq code as key, '' as value
  print "Sorting BLAST results\n";
  sort_blast_results(\%seq, \%blast, \%css, $blast_out);
  
  if (0)
  {
  open_file_out($temp_folder.'/blast1.txt', 'BL1');
  print BL1 Dumper (\%blast);
  close BL1;
  }

      # delete_multihit_css from the %blast hash and from %seq
      # print multihit_css_file;
  print "Remove multihit sequences\n";
  treat_multihit_css(\%blast, \%css, \%seq, $multihit_css_file, \%seq_count);
  
  if (0)
  {
  open_file_out($temp_folder.'/blast2.txt', 'BL2');
  print BL2 Dumper (\%blast);
  close BL2;
  }
  
      # for each remaining blast line align the two sequence in the rigth orientation, and determine %ident for the flanking region
      # if ident is bigger than limit, pool sequences into, one group
   print "Identifying sequence groups for consensus sequences\n";
   my %gr_seq = (); # group_id as key, [seq_code;orientation (1/+1)] as value
   my %seq_gr = (); # seq_id as key, group_id as value

  calc_perc_ident_flanking_make_big_group(\%blast, \%seq, \%gr_seq, \%seq_gr, $param{ident_limit}, $temp_folder, $param{clust_path});

    print "Making consensus sequences\n";
  # conensus file sequences and subcons sequences are with gaps (aligned) 
  # subcons sequences are deleted from %seq
  make_cons_file_1(\%gr_seq, \%seq_gr, \%seq, $param{prop_maj}, $cons_subs_file, $param{clust_path}, $temp_folder, \%seq_count);

  my $find_ms2 = 'perl find_and_mask_ms2.pl 1000000 5 5 5 5 5 1 1 "'.$cons_subs_file.'" "'.$temp_folder.'" cons_subs\.fas 1 1 1';
    #  USAGE: $min1 $min2 $min3 $min4	$min5	$min6	$del_prev_mask $keep_non_ms $folder/$file	$outfolder $file_motif $write_seq_file $write_fas_file $one_file;
	print LOG "\n$find_ms2\n";
  print "Check polymorphism in consensus sequences\n";
  system $find_ms2;
  
  my $ms_file = $temp_folder.'/'.$file_root.'_cons_subs.seq';
  my $masked_file = $temp_folder.'/'.$file_root.'_cons_subs_mask.fas';
  my %cons_poly = ();
  check_polymorphism(\%cons_poly, $ms_file, $masked_file);
#  print_hachage(\%cons_poly);
  
   # replace non_masked sequences of cons subs file by MS masked (4 repeat, but not homopolymer)
   # write consensus file with ms masked and polymorphism info (motif_first_last pas in the aligned cons seq)
  write_consensus_file_with_polymorphism_info(\%cons_poly, $consensus_file, $cons_subs_file, $masked_file);


  # identify regions of %gr_sequences that have hit to other sequences from %blast; regions of hit prined in lc;
  # make gr_list.fas
  my $gr_file_temp = $temp_folder.'/'.$file_root.'_gr.fas'; # MS and homopolymer masked; $gr_file hit to other sequeces are masked
  treat_group_seq(\%seq, \%blast, $gr_file, $gr_file_temp, \%seq_count);
  
  # delete %blast OR make consensus more precise... TO DO
  
 
# BLast consensus.fas againts gr_list.fas
  print "Blasting consensus sequences against grouped sequences\n";
  my $blast_out_cons_gr = $temp_folder.'/cons_gr_blast.tbl';
  my $db_group_file = $outfolder.'_'.$file_root.'_gr.fas'; 
  copy_one_file($temp_folder, $file_root.'_gr.fas', $db_group_file, $param{syst}); # copy $gr_file_temp into working dir folder to be able to make a db

  call_blast_query_to_db_soft_mask($consensus_file, $db_group_file, $blast_out_cons_gr, $e, $param{blast_path}, $blast_version, $param{syst});
  delete_tag_adapt_files($db_group_file, $param{syst});

# if hit of cons, print it as grouped in class file, else print it as unique cons

  # remove cons sequences from consensus file if they had BLAST hit to grouped sequences; and add them to gr_seq
  print "Moving consensus sequences with partial hits to grouped sequences into grouped file\n";
  identify_grouped_cons_seq($blast_out_cons_gr, $cons_subs_file, $consensus_file, $gr_file, \%seq_count);
  
  pool_files($unique_file, $consensus_file, $pipe3_input_file);
  
  print_hachage(\%seq);
  } # fin if $make_cons
  
  print LOG "\n\n";
  foreach my $code (sort keys %seq_count)
  {
    print LOG "$code $seq_count{$code}\n";
  }
  my $time_end = time;
  my $time_run = $time_end - $time_beg;
  print LOG "\nThe run took $time_run s.\n";
  
  move_files_local($outfolder_spec, $pipe2_folder, $param{syst}); # move files from working folder to pipe2 folder
 
#if ($param{del_files})
#{
#  delete_folder($outfolder_spec, $param{syst});
#}


} # end if file motif
} # end foreach my $file
close LOG;

if ($param{del_files} ==1)
{
  delete_folder($outfolder, $param{syst});
}
print "\nThe anlysis is finished\n";

exit;

############################################################

sub sort_blast_results
{
my ($seq, $blast, $css, $blast_out) =@_;

# %seq = (); # sequences that had autohit and to other sequences
# %blast = (); # hash of anonumous hash $blast{code1}{code2}=>[blast line]
# does not contain autohits; only one hit is recorded in case of reciproc hits; 
#several hits between the two same sequences is marked as css
# %css =(); # seq code as key, '' as value

  open_file_in($blast_out, 'BL');
  while (my $line = <BL>)
  {
    my @line = split(' ', $line);
    
    unless ($line[0] eq $line[1])
    {
    if (exists $$seq{$line[0]} and exists $$seq{$line[1]}) 
    {
      if (exists $$blast{$line[0]} and exists $$blast{$line[0]}{$line[1]}) # hit between the two same sequences already tested => css_multihit
      {
        $$blast{$line[0]}{$line[1]} = 'css';
        $$css{$line[0]} = '';
        $$css{$line[1]} = '';     
      }
      elsif (exists $$blast{$line[1]} and exists $$blast{$line[1]}{$line[0]}) # reciproc hit already tested
      {
        next;
      }
      else 
      {
        $$blast{$line[0]}{$line[1]} = [$line[2], $line[6], $line[7], $line[8], $line[9]];
      }
    }
    }

    
  } # end while
} # end sub

########################################################

sub delete_multihit_css_make_gr_seqlist
{
# delete_multihit_css from the %blast hash
# make a hash with the list of seq code that have genuine hit to other sequences and are not css.
# the case of very few sequences if their only valid hit is towards a css_multihit sequence, they are classed as unique, sincs tjeir css pair is deleted
my ($blast, $css, $autohit) = @_;
my %gr_list = ();

foreach my $code1 (keys %$blast)
{
  if (exists $$css{$code1}) # if code1 css delete all blast hits with code1
  {
    delete$$blast{$code1};
  }
  elsif(exists $$autohit{$code1})
  {
    foreach my $code2 (keys %{$$blast{$code1}})
    {
      if (exists $$css{$code2}) # if code2 css but not code1 delete specific code2 blast hits
      {
        delete$$blast{$code1}{$code2};   
      }
      elsif (exists $$autohit{$code2})
      {
        $gr_list{$code2} = '';
      }
      else # code2 is css no_hit
      {
        delete$$blast{$code1}{$code2};         
      }
    }
    
    if (scalar(keys %{$$blast{$code1}}) == 0) # if all hits to code1 are deleted
    {
      delete$$blast{$code1};
    }
    else
    {
        $gr_list{$code1} = '';
    }
  }
  else # code1 is css no_hit
  {
    delete$$blast{$code1};    
  }
}

return %gr_list;  
} # end sub


##############################################################

sub calc_perc_ident_flanking_make_big_group
{
my ($blast, $seq, $gr_seq, $seq_gr, $ident_limit, $temp_folder, $clust_path) =@_;

 #  %gr_seq = (); # group_id (first seq in group) as key, {seq_code =>orientation (1/+1, relative to the group name seq)} as value
 #  %seq_gr = (); # seq_id as key, group_id as value

foreach my $code1 (keys %$blast)
{
  foreach my $code2 (keys %{$$blast{$code1}})
  {
 #   print "$code1; $code2;\n";

   my $orient = determine_orient($$blast{$code1}{$code2});
    # if identity is calculated, BLASt resukts are replaced by flanking ident, oriens seq1 align, seq2 align
  
  unless (exists $$seq{$code2})
  {
    print STDERR "$code2\n";
    print STDERR "$code1 $code2\n";
#    print STDERR Dumper "$$blast{$code1}\n";
  }
   unless (exists $$seq{$code1})
  {
    print STERR "$code1\n";
    print STERR "$code1 $code2\n";
#    print STERR Dumper "$$blast{$code1}\n";
  }
   my $ident = determine_ident($$blast{$code1}{$code2}, $orient, $ident_limit, $$seq{$code1}, $$seq{$code2}, $temp_folder, $clust_path);
   
 #   print "$$blast{$code1}{$code2}[0], $$blast{$code1}{$code2}[1]\n";
 #   print "$$blast{$code1}{$code2}[2]\n$$blast{$code1}{$code2}[3]\n\n";

    if ($ident >= $ident_limit)
    {
      if (exists $$seq_gr{$code1} and $$seq_gr{$code2} )
      {
        if (not ($$seq_gr{$code1} eq $$seq_gr{$code2}))
        {
          merge_groups1($gr_seq, $seq_gr, $code1, $code2, $orient);
        }
      }
      elsif (exists $$seq_gr{$code1})
      {
        add_to_group($gr_seq, $seq_gr, $code2, $code1, $orient);
      }
      elsif (exists $$seq_gr{$code2})
      {
        add_to_group($gr_seq, $seq_gr, $code1, $code2, $orient);
      }
      else
      {
        make_new_group($gr_seq, $seq_gr, $code1, $code2, $orient);
      }
    } # end if ident > limit
  } # end foreach $code2
} # end foreach $code1

#print Dumper $gr_seq;
} # end sub

#############################################

sub determine_ident
{
my ($blast, $orient, $ident_limit, $seq1, $seq2, $temp_folder, $clust_path) = @_;
# $blast is a ref to an anonymous table with [blast_ident, qfirst, qlast, sfirst, slast]
my %seq_cut = (); # keys: s1before, s1overlap, s1after, s2before, s2overlap, s2after

my $ident = 0;
my $overhang = 50;

if ($$blast[0] >= ($ident_limit - 5)) # examine hits only if the balst identity is at most 5% smaller than limit
{
    # length of the overlaping sequence that is not alingned by BLAST
    # determine %$seq_cut
  my $non_align = determine_non_aligned_length($blast, $orient, $seq1, $seq2, \%seq_cut); # length of the overlaping sequence that is not alingned by BLAST
  if ($non_align < $overhang) # examine hits only if the non_blast_aligned is smaller than 50 bases 
  {
    my %seq1_mspos = determine_ms_pos($seq_cut{s1overlap});
    my %seq2_mspos = determine_ms_pos($seq_cut{s2overlap});
    my %seq_align = align_2seq($seq_cut{s1overlap}, $seq_cut{s2overlap}, $clust_path, $temp_folder);
#    print_hachage(\%seq_align);
    $ident= calc_ident(\%seq_align, \%seq1_mspos, \%seq2_mspos);
#    print_hachage(\%seq_align);
#    print TMP "$overhang;$$blast[0];$ident\n";
      # replace the BLAST results by the %identity on the flanking region, $orientation, seq1 align; seq2 align
    if ($ident >= $ident_limit)
    {
      add_seq_cut_to_blast($blast, \%seq_align, \%seq_cut, $ident, $orient); 
    }
  }
}

return $ident;   
}
#############################################

sub determine_orient
{
my ($blast) = @_;
# $blast is a ref to an anonymous table with [blast_ident, qfirst, qlast, sfirst, slast]
my $orient = 1;

if ($$blast[3] > $$blast[4])
{
  $orient = -1;
}

return $orient;
}
#############################################
sub determine_non_aligned_length
{
my ($blast, $orient, $seq1, $seq2, $seq_cut) = @_;
# $blast is a ref to an anonymous table with [blast_ident, qfirst, qlast, sfirst, slast]


my $l1 = length$seq1;
my $l2 = length$seq2;
my $beg = $$blast[1] -1;
my $end = ($l1) - $$blast[2];
if ($orient == 1)
{
  if ($$blast[3] <= $beg)
  {
     $beg = $$blast[3] -1;
  }
  my $temp = ($l2) - $$blast[4];
  if ( $temp < $end)
  {
     $end = $temp;
  }

}
else
{
  my $temp1 = ($l2) - $$blast[3];

  if ($temp1 < $beg)
  {
     $beg = $temp1;
  }
  if ($$blast[4] <= $end)
  {
     $end = $$blast[4] -1;
  }
#   print "@$blast; $l1; $l2; $beg; $end\n";

}
my $non = $beg + $end;

my $s1before = $$blast[1] - $beg;
my $s1overlap = $$blast[2] - $$blast[1] +1 + $non;
my $s1after = $l1 - $$blast[2] - $end;
my $s2before = '';
my $s2overlap = '';
my $s2after = '';

if ($orient == 1)
{
  $s2before = $$blast[3] - $beg;
  $s2overlap = $$blast[4] - $$blast[3] +1 + $non;
  $s2after = $l1 - $$blast[4] - $end;
}
else
{
  $s2before = $l2 - $$blast[3] - $beg;
  $s2overlap = $$blast[3] - $$blast[4] +1 + $non;
  $s2after = $$blast[4] - $end;
  $seq2 = reverse_complement_loc($seq2);
}

#print "$seq1\n";
$$seq_cut{s1before} = substr($seq1, 0, $s1before, '');
$$seq_cut{s1overlap} = substr($seq1, 0, $s1overlap, '');
$$seq_cut{s1after} = $seq1;
$$seq_cut{s2before} = substr($seq2, 0, $s2before, '');
$$seq_cut{s2overlap} = substr($seq2, 0, $s2overlap, '');
$$seq_cut{s2after} = $seq2;

#unless ($orient ==1)
#{
#print "$beg;$end;\n1:$$seq_cut{s1overlap}\n2:$$seq_cut{s2overlap}\n\n";
#}

return $non;
}

######################################################

sub determine_ms_pos
{
my ($seq) = @_;
my %seq_mspos = ();
my @seq = split('', $seq);
for(my $i=0; $i< scalar@seq; ++$i)
{
  if ($seq[$i] =~ /[atgc]/)
  {
    $seq_mspos{$i} = '';
  }
}

return %seq_mspos;
}
######################################################

sub align_2seq
{
my ($seq1, $seq2, $clust_path, $temp_folder) = @_;
my $file = $temp_folder.'/clustal.fas';
my $out_cl = $temp_folder.'/clustal.gde';
my $log = $temp_folder.'/logfile';
open_file_out($file, 'IN');
print IN ">seq1\n$seq1\n>seq2\n$seq2\n";
close 'IN';

system '"'.$clust_path.'clustalw2" -INFILE='.$file.' -QUIET -OUTPUT=GDE >'.$log;

open_file_in($out_cl, 'AL');
my $code = '';
my %seqa = ();
while (my $line = <AL>)
{
  chomp $line;
  if ($line =~ /\#/)
  {
     $code = $line;
     $code =~ s/\#//;
  }
  else
  {
    $seqa{$code} .= $line;
  }
}
close AL;
return %seqa;

}

###################################################################
sub calc_ident
{
my ($seq_align, $seq1_ms, $seq2_ms) = @_;
my @seq1 = split('', $$seq_align{seq1});
my @seq2 = split('', $$seq_align{seq2});
my $s1 = -1; 
my $s2 = -1; 
my $flank_l = 0;
my $mism = 0;
for (my $i = 0; $i<scalar@seq1; ++$i)
{
  unless($seq1[$i] eq '-')
  {++$s1;}
  unless($seq2[$i] eq '-')
  {++$s2;}
  if (exists $$seq1_ms{$s1} or exists $$seq2_ms{$s2})
  {next;}
  
  ++$flank_l;
  unless($seq1[$i] eq $seq2[$i])
  { ++$mism;}
}
my $ident = 0;
if ($flank_l >20)
{
$ident = int((($flank_l-$mism)/$flank_l)*100);
}
return $ident;
}
###################################################################
sub add_seq_cut_to_blast
{
my ($blast, $seq_align, $seq_cut, $ident, $orient) = @_;
# $blast is a ref to an anonymous table with [blast_ident, qfirst, qlast, sfirst, slast]
# This is changed to [flank_ident, orient, qalig, salig]
my $seq1 = '';
my $seq2 = '';

# write sequences before the overlap
if ($$seq_cut{s1before} eq '')
{
  unless($$seq_cut{s2before} eq '')
  {
    $seq2 = $$seq_cut{s2before};
    $seq1 = '-' x (length $$seq_cut{s2before});
  }
}
else
{
    $seq1 = $$seq_cut{s1before};
    $seq2 = '-' x (length $$seq_cut{s1before});
}

# write aligned sequences in the overlap
    $seq2 .= $$seq_align{seq2};
    $seq1 .= $$seq_align{seq1};

# write sequences after the overlap
if ($$seq_cut{s1after} eq '')
{
  unless($$seq_cut{s2after} eq '')
  {
    $seq2 .= $$seq_cut{s2after};
    $seq1 .= '-' x (length $$seq_cut{s2after});
  }
}
else
{
    $seq1 .= $$seq_cut{s1after};
    $seq2 .= '-' x (length $$seq_cut{s1after});
}

@$blast = ($ident, $orient, $seq1, $seq2);
}
#################################################################
sub merge_groups1
{
 my ($gr_seq, $seq_gr, $code1, $code2, $orient) =@_;
 
 my $gr = $$seq_gr{$code1};
 my $gr_to_del = $$seq_gr{$code2};
 my $code1_orient = $$gr_seq{$gr}{$code1};

# print Dumper $gr_seq;
 foreach my $code (keys %{$$gr_seq{$gr_to_del}})
 {
  my $o = $$gr_seq{$gr_to_del}{$code};
  $$gr_seq{$gr}{$code} = $o * $code1_orient;
  $$seq_gr{$code} = $gr;
 }
  delete $$gr_seq{$gr_to_del};
 #  print Dumper $gr_seq;

}
#############################################
sub add_to_group
{
 my ($gr_seq, $seq_gr, $new_code, $old_code, $orient) =@_;
# old_code is already in a group
 
 $$seq_gr{$new_code} = $$seq_gr{$old_code};

# print "$new_code, $old_code, $orient\n";
# print Dumper $$gr_seq{$$seq_gr{$old_code}};
 
  my $old_code_orient = $$gr_seq{$$seq_gr{$old_code}}{$old_code};
 $orient = $orient * $old_code_orient;
 $$gr_seq{$$seq_gr{$old_code}}{$new_code} = $orient; 
# print Dumper $$gr_seq{$$seq_gr{$old_code}};
# print "*********************\n";
}

#############################################

sub make_new_group
{
 my ($gr_seq, $seq_gr, $code1, $code2, $orient) =@_;
 #  %gr_seq = (); # group_id (first seq in group) as key, {seq_code =>orientation (1/+1, relative to the group name seq)} as value
 #  %seq_gr = (); # seq_id as key, group_id as value

#print "$code1\n$code2\n";
#print Dumper $gr_seq;
 $$seq_gr{$code1} = $code1;
 $$seq_gr{$code2} = $code1;
 $$gr_seq{$code1}{$code2} = $orient;
 $$gr_seq{$code1}{$code1} = 1;
# print Dumper $gr_seq;

}

#################################################
sub make_cons_file_1
{
my ($gr_seq, $seq_gr, $seq, $prop_maj, $cons_file_subseq, $clust_path, $temp_folder, $seq_count_hash) = @_;

my $file = $temp_folder.'/clustal.fas';
my $out_cl = $temp_folder.'/clustal.gde';
my $log = $temp_folder.'/logfile';

open_file_out($cons_file_subseq, 'CONSUB');

system '"'.$clust_path.'clustalw2" -INFILE='.$file.' -QUIET -OUTPUT=GDE >'.$log;

$$seq_count_hash{'Number of sequences pooled into consensus sequences:'} = 0;

my $gr_count = 0;
foreach my $gr (sort keys %$gr_seq)
{
  ++$gr_count;
  open_file_out($file, 'IN');
  my $seq_count = 1; # code 'seqx' as a key; real code as a value; for clustal short coe is necessary (seqx), but keep track of real codes
  my %code_corr = ();
  foreach my $code (sort keys %{$$gr_seq{$gr}})
  {
    if ($$gr_seq{$gr}{$code} == -1)
    {
      $$seq{$code} = reverse_complement_loc($$seq{$code});
    }
    print IN ">seq_$seq_count\n$$seq{$code}\n";
    $code_corr{'seq_'.$seq_count} = $code;
    ++$seq_count;
  }
  close 'IN';

  system '"'.$clust_path.'clustalw2" -INFILE='.$file.' -QUIET -OUTPUT=GDE >'.$log;
  open_file_in($out_cl, 'AL');
  my $code = '';
  my %seqa = ();
  while (my $line = <AL>)
  {
    chomp $line;
    if ($line =~ /\#/)
    {
      $code = $line;
      $code =~ s/\#//;
      $code = $code_corr{$code};
    }
    else
    {
      $seqa{$code} .= $line;
    }
  }
  close AL;
  my $cons = make_cons(\%seqa, $prop_maj);
  
  --$seq_count;
  print CONSUB ">cons_gr$gr_count";
  print CONSUB "_$seq_count\n$cons\n";
  foreach my $code (sort keys %seqa)
  {
    print CONSUB ">$code orientation:$$gr_seq{$gr}{$code}\n$seqa{$code}\n";
    delete $$seq{$code};
    ++$$seq_count_hash{'Number of sequences pooled into consensus sequences:'};
  }

}
close CONSUB;

}

#######################################
sub make_cons
{
my ($seq_align, $prop_maj) = @_;
my @codes = sort keys %$seq_align;
my $cons = '';
my %seq_temp = %$seq_align;

# replace end gaps by '*';
foreach my $code (@codes)
{
  my $l = '';
  my $temp = '';
  if ($seq_temp{$code} =~  s/^-+//)
  {
    $l = length $&;
    $temp = '*' x $l;
    $seq_temp{$code} = $temp.$seq_temp{$code};
  }
  if ($seq_temp{$code} =~  s/-+$//)
  {
    $l = length $&;
    $temp = '*' x $l;
    $seq_temp{$code} .= $temp;
  }
}


for(my $i= 0; $i < length($seq_temp{$codes[0]}); ++$i)
{
  my %bases = ('A' => 0,
               'T' => 0,
                'C' => 0,
                'G' => 0,
                '-' => 0,
                '*' => 0);
                
  foreach my $code (@codes)
  {
    my $base = substr($seq_temp{$code}, $i, 1);
    ++$bases{uc$base};
  }

  my $n = scalar@codes - $bases{'*'};
  delete $bases{'*'};
  my @count = sort values %bases;
  if ($count[-1]/$n >= $prop_maj)
  {
      foreach my $b (keys %bases)
      {
        if ($bases{$b} == $count[-1])
        {
          $cons .= $b;
          last;
        }
      }
  }
  else 
  {$cons .= 'N';}
}
return $cons;
}

#################################################

sub treat_group_seq
{
my ($seq, $blast, $gr_file, $gr_file_temp, $seq_count) = @_;
  # identify regions of %gr_sequences that have hit to other sequences from %blast
  # print %gr_list into class file with regions of hit prined in lc; delete gr_list from %seq
  # make gr_list.fas
  
  # mask hit regions in all grouped sequences; delete %blast lines with grouped sequences
  my %seq_mask = ();
  foreach my $code1 (keys %$blast)
  {
    foreach my $code2 (keys %{$$blast{$code1}})
    {
      my $bool = 0;
      my $seq_temp = '';
      if (exists $$seq{$code1})
      {
        if (exists $seq_mask{$code1})
        {$seq_temp = $seq_mask{$code1};}
        else
        {$seq_temp = $$seq{$code1};}
        
        $seq_mask{$code1} = mask_interval($seq_temp, $$blast{$code1}{$code2}[1], $$blast{$code1}{$code2}[2]);
        $bool = 1;
      }
      if (exists $$seq{$code2})
      {
        if (exists $seq_mask{$code2})
        {$seq_temp = $seq_mask{$code2};}
        else
        {$seq_temp = $$seq{$code2};}
 
         $seq_mask{$code2} = mask_interval($seq_temp, $$blast{$code1}{$code2}[3], $$blast{$code1}{$code2}[4]);   
         $bool = 1;    
      }
      if ($bool)
      {
        delete $$blast{$code1}{$code2};
      }
    }
  }
  
    # print %gr_list into class file with regions of hit prined in lc; delete gr_list from %seq
     # make gr_list.fas

  open_file_out($gr_file, 'GR');
  open_file_out($gr_file_temp, 'GRT');
  $$seq_count{'Number of grouped sequenes (without consensus):'} = 0;
  foreach my $code (sort keys %$seq)
  {
    print GRT ">$code\n$$seq{$code}\n";
    ++$$seq_count{'Number of grouped sequenes (without consensus):'};
    if (exists $seq_mask{$code})
    {
      print GR ">$code\n$seq_mask{$code}\n";
    }
    else # the only blast hit of the sequence was to nohit_css sequence and thus elimnated earlier 
    {
      print GR ">$code\n$$seq{$code}\n";    
    }

    delete $$seq{$code};
  }
  close GR;
  close GRT;
  
}
 
######################################################

sub mask_interval
{
my ($seq, $beg, $end) = @_;
if ($beg > $end)
{
  my $temp = $beg;
  $beg = $end;
  $end = $temp;
}
#print "$beg, $end\n $seq\n";
my $frag = lc substr($seq, $beg-1, $end-$beg +1);
substr($seq, $beg-1, $end-$beg +1, $frag);
#print "$seq\n";

return $seq;
}

######################################################

sub identify_grouped_cons_seq
{
my ($blast_out_cons, $cons_subs_file, $cons_file, $gr_file, $seq_count) = @_;

open_file_in($blast_out_cons, 'BL');
my %blast = ();

# identify hit region for each grouped consensus sequences
while (my $line = <BL>)
{
  my @line = split(' ', $line);

  unless(exists $blast{$line[0]})
  {
    $blast{$line[0]} = [$line[6], $line[7]];
    next;      
  }
  
    if ($blast{$line[0]}[0] > $line[6])
    {
      $blast{$line[0]}[0] = $line[6];
    }
    if ($blast{$line[0]}[1] < $line[7])
    {
      $blast{$line[0]}[1] = $line[7]; 
    }   
}
close BL;


# delete grouped consensus sequences from cons_file
# add grouped cons seq to gr_file;
$$seq_count{'Number of unique consensus sequences:'} = 0;
$$seq_count{'Number of grouped consensus sequences:'} = 0;
open_file_in($cons_file, 'CONS');
my @cons = <CONS>;
close CONS;
open_file_out($cons_file, 'COOUT');
unless(open('GR', ">>$gr_file"))
{print "Cannot open $gr_file";}

my $code = '';
my $code_line = ''; # kep the definition line with polymorphism info
foreach my $line (@cons)
{
  chomp $line;
  if ($line =~ />/)
  {
  $code = $line;
  $code =~ s/>//;
  $code =~ s/\s.*//;
  $code_line = $line;
  }
  else
  {
    if (exists $blast{$code})
    {
      my $fr = substr($line, $blast{$code}[0]-1, $blast{$code}[1]-$blast{$code}[0] +1);
      substr($line, $blast{$code}[0]-1, $blast{$code}[1]-$blast{$code}[0] +1, lc$fr);
      print GR "$code_line\n$line\n";
      ++$$seq_count{'Number of grouped consensus sequences:'};
    }
    else
    {
      print COOUT "$code_line\n$line\n";
      ++$$seq_count{'Number of unique consensus sequences:'};
    }
  }
}
close COOUT;
close GR;
@cons = ();

if (0)
{
open_file_in($cons_subs_file, 'COS');
my $cons_code= '';
my $seq_code = '';
while (my $line = <COS>)
{
  chomp $line;
  if($line =~ />cons/)
  {
    $cons_code = $line;
    $cons_code =~ s/>//;
    $seq_code = '';
  }
  elsif ($line =~ />/)
  {
    $seq_code = $line;
    $seq_code =~ s/>//; 
  }
  else
  {
    if (exists $blast{$cons_code})
    {
      if ($seq_code eq '')
      {
        $line =~ s/-//g;
        my $fr = substr($line, $blast{$cons_code}[0]-1, $blast{$cons_code}[1]-$blast{$cons_code}[0] +1);
        substr($line, $blast{$cons_code}[0]-1, $blast{$cons_code}[1]-$blast{$cons_code}[0] +1, lc$fr);
        print GR ">$cons_code\n$line\n";
      }
      else
      {
        $line = uc $line;
        print GR ">$cons_code";
        print GR "_$seq_code\n$line\n";
      }
    }
  }
}
close COS;
close GR;
}

}
######################################################

sub check_polymorphism
{
my ($cons_poly, $ms_file, $masked_file) = @_;

my %seq = read_hach_from_fasta_code_wo_space($masked_file);
open_file_in($ms_file, 'IN');
my $title = <IN>;
my %cons = (); # hach of hach of tables $cons{$cons_code}{$ms_code}[ms_mot, beg, end, rep_number_seq1...]
my $cons_code = '';
while (my $line = <IN>)
{
  chomp $line;
  my @line = split(';', $line);
  if($line[0] =~ /cons/) # collect info on Ms position in the aligned cons seq
  {
    for (my $i = 3; $i< scalar@line; $i = $i + 4)
    {
      $cons_code = $line[0];
      if (($line[$i+1] < length$line[$i]) or (($line[2]- $line[$i+2]) <length$line[$i])) # if ms starts at the begining or end at the end of the cons seq.
      { next; }
      $cons{$line[0]}{($i+1)/4} = [$line[$i], $line[$i+1], $line[$i+2]];
    }
  }
  else
  {
     for (my $i = 3; $i< scalar@line; $i = $i + 4)
    {
      $seq{$line[0]} =~ /^-+/;
      my $gap_beg = length $&;
      $seq{$line[0]} =~ /-+$/;
      my $gap_end = length $&;
      if ((($line[$i+1]-$gap_beg) < length$line[$i]) or $line[2]-$line[$i+2]-$gap_end < length$line[$i]) # if ms is within one motif length of the begining or the end of the seqeunce (after eliminating end gaps)
      {next;}
      check_overlap($line[$i+1], $line[$i+2], $line[$i+3], $cons{$cons_code}); # if ms overlaps with one of the MS in cons seq, add repeat number to $cons{$cons_code}{$ms_code}[ms_mot, beg, end, rep_number_seq1...]
    }   
  }
}
close IN;

foreach my $code (keys %cons)
{
  foreach my $ms (keys %{$cons{$code}})
  {
    my $polymorph = 0;
    for (my $j = 4; $j < scalar@{$cons{$code}{$ms}}; ++$j)
    {
      if ($cons{$code}{$ms}[3] != $cons{$code}{$ms}[$j])
      {
        $polymorph = 1;
        last;
      }
    }
    if ($polymorph)
    {
      # adjust MS beg and Ms end positions in the consensus sequence WO gaps (tha actual values are with gaps)
      my $adjusted_pos = ''; # msmotif_beg_end
      adjust_ms_pos(\$adjusted_pos, $cons{$code}{$ms}[0], $cons{$code}{$ms}[1], $cons{$code}{$ms}[2], $seq{$code});
#      $$cons_poly{$code} .= $cons{$code}{$ms}[0].'_'.$cons{$code}{$ms}[1].'_'.$cons{$code}{$ms}[2].';';
      $$cons_poly{$code} .= $adjusted_pos.';';
    }
  }
}

%cons = ();
%seq = ();
#print Dumper %cons;

}
######################################################

sub adjust_ms_pos
{
 my ($adjusted_pos_ref, $mot, $ms_beg, $ms_end, $seq_align) =@_;
 
 my $beg = substr($seq_align, 0, $ms_beg-1);
 $beg =~ s/-//g;
 $ms_beg = (length $beg) +1;
 
 my $end = substr($seq_align, 0, $ms_end-1);
 $end =~ s/-//g;
 $ms_end = (length $end) +1;
 $$adjusted_pos_ref = $mot.'_'.$ms_beg.'_'.$ms_end;
 
}
#########################################################
sub check_overlap
{
my ($beg, $end, $rep_n, $cons_ref) =@_;
foreach my $ms (keys %$cons_ref)
{
  if (($$cons_ref{$ms}[1] <= $beg and $beg <= $$cons_ref{$ms}[2]) or ($$cons_ref{$ms}[1] <= $end and $end <= $$cons_ref{$ms}[2]) or ($$cons_ref{$ms}[1] >= $beg and $end >= $$cons_ref{$ms}[2]))
  {
    push(@{$$cons_ref{$ms}}, $rep_n);
  }
}
}

######################################################

sub write_consensus_file_with_polymorphism_info
{
my ($cons_poly, $consensus_file, $cons_subs_file, $masked_file) = @_;
open_file_out($consensus_file, 'CON');
open_file_out($cons_subs_file, 'SUB');
open_file_in($masked_file, 'MASK');
my $code = '';
while(my $line = <MASK>)
{
  print SUB "$line";
  if ($line =~ />cons/)
  {
    $code = $line;
    chomp $code;
    $code =~ s/>//;
  }
  elsif($line =~ />/)
  {$code = '';}
  elsif($code)
  {
    print CON ">$code";
    if (exists $$cons_poly{$code})
    {
      print CON " $$cons_poly{$code}";
    }
    print CON "\n";
    
    $line =~ s/-//g;
    print CON "$line";
  }
}
close MASK;
close CON;
close SUB;
}

#####################################################

sub make_unique_only
{
my ($seq, $unique_file, $nohit_css_file, $blast_out, $seq_count) =@_;
#make file with sequenes that had only an autohit, but nothing else
#make file with sequenes that had did not had autohit
# delete all these sequences from %$seq

my %hit = (); #codes of all sequences that had non-autohit
my %autohit = (); # codes of all sequences that had autohit
open_file_in($blast_out, 'BL');
while(my $line = <BL>)
{
  my @line = split(' ', $line);
  if($line[0] eq $line[1])
  {
    $autohit{$line[0]} = '';
  }
  else
  {
    $hit{$line[0]} = '';
    $hit{$line[1]} = '';
  }
}
close BL;

open_file_out($unique_file, 'OUT');
open_file_out($nohit_css_file, 'NOHIT');

$$seq_count{'Number of unique sequences:'} = 0;
$$seq_count{'Number of sequences without autohit (nohit_css):'} = 0;
foreach my $code (sort keys %$seq)
{
  if (exists $autohit{$code})
  {
    unless(exists $hit{$code})
    {
      print OUT ">$code\n$$seq{$code}\n";
      delete $$seq{$code};
      ++$$seq_count{'Number of unique sequences:'};
    }
  }
  else
  {
    print NOHIT ">$code\n$$seq{$code}\n";  
    delete $$seq{$code};
    ++$$seq_count{'Number of sequences without autohit (nohit_css):'};
  }
}
close OUT;
close NOHIT;

%hit = ();
%autohit = ();

}

########################################################

sub treat_multihit_css
{
my ($blast, $css, $seq, $multihit_css_file, $seq_count) = @_;
# delete_multihit_css from the %blast hash and from %seq

foreach my $code1 (keys %$blast)
{
  if (exists $$css{$code1}) # if code1 css delete all blast hits with code1
  {
    delete$$blast{$code1};
  }
  else
  {
    foreach my $code2 (keys %{$$blast{$code1}})
    {
      if (exists $$css{$code2}) # if code2 css but not code1 delete specific code2 blast hits
      {
        delete$$blast{$code1}{$code2};   
      }
    }
    if (scalar(keys %{$$blast{$code1}}) == 0) # if all hits to code1 are deleted
    {
      delete$$blast{$code1};
    }
  }
}


# delete_multihit_css from %seq
# print multihit_css_file;
open_file_out($multihit_css_file, 'MULT');
$$seq_count{'Number of multihit css sequences:'} = 0;
foreach my $code (keys %$seq)
{
  if (exists $$css{$code})
  {
    print MULT ">$code\n$$seq{$code}\n";
    delete $$seq{$code};
    ++$$seq_count{'Number of multihit css sequences:'};
  } 
}

close MULT;

}

########################################################

sub define_param_number_pipe2
{
  my %param_list = (
      1=> ['syst', 'Operating system (win/linux):'], 
      2=> ['project_folder', 'Project folder:'], 
      3=> ['make_cons', 'Make consensus sequences (YES=1/NO=0):'],
      4=> ['ident_limit', 'Minimum % of identity between sequences of a contig (80-100):'], 
      5=> ['prop_maj', 'Proportion of sequences that must have the same base at a site to accept it as a consensus (0.5-1):' ], 
      6=> ['clust_path', 'Pathway to CLUSTALW2 executables:'], 
      7=> ['blast_path', 'Pathway to BLAST executables:'], 
#      8=> ['blast_version', 'BLAST version (blast/blast+):'],
      8=> ['del_files', 'Delete intermediate files (YES=1/NO=0):']
      );
  return %param_list;
}

##################################################################

sub define_param_limits_pipe2
{
my ($param) = @_;
#hash of array [type(list, real, folder), error message, ['list of values/lower_upper limits)']
  my %param_limits = (
      'syst'=> ['list', 'Operating system must be win or linux', ['win', 'linux']], 
      'project_folder' => ['folder', "Cannot open the Project folder $$param{project_folder}"], 
      'make_cons'=> ['list', 'make_cons must be 1 or 0', [0,1]],
      'ident_limit'=> ['real', 'ident_limit must be between 80-100', [80,100]], 
      'prop_maj'=> ['real', 'prop_maj must be between 0.5-1', [0.5,1]], 
      'clust_path'=> ['folder', "Cannot open $$param{clust_path} folder"], 
      'blast_path'=> ['folder', "Cannot open $$param{blast_path} folder"], 
#      'blast_version'=> ['list', 'blast_version must be blast or blast+', ['blast','blast+']],
      'del_files'=> ['list', 'del_files must be 0 or 1', [0,1]]
      );
}
##################################################################
sub delete_tag_adapt_files
{
my ($file, $syst) = @_;
if ($syst eq 'win')
{
  system 'del '.$file.'*';
}
else
{
  system 'rm '.$file.'*';
}
}
