#!/usr/bin/perl -w


  use warnings;
  use strict;
  use subprogramQDD;
#  use Data::Dumper;
  
  
#INPUT: fasta file
# output: 

# if $direct_par_def = 0 (command line menu) set_file is read and the user is asked if the settings are correct and can change them 
# if $direct_par_def = 1 all parameters are defined in the program; 
# if $direct_par_def = 2 the set_file is read and the user cannot change the parameters (option for running behind GUI)

my $direct_par_def = 0;

my $set_file = 'set_pipe4.txt'; 
my $set_file_default = 'set_pipe4_default.txt';
my $blast_prog = 'blastn';
my $db   = 'nr';
my $e_val= '1e-20';

print_heading();

my %param = (
  syst => '',  
  project_folder => '',
  del_files => '',
);
my %param_list = define_param_number_pipe4(); # $param_list{1}[syst, 'Operating system (win/linux):'], ...
my %param_limits = define_param_limits_pipe4(\%param); #hash of array $param_limits{syst}[type(list, real, folder), error message, ['list of values/lower_upper limits)']

if ($direct_par_def == 1) # define parameteres values in the script
{
  $param{syst} = 'win';  
  $param{del_files} = 1;
  $param{project_folder} = 'd:/test with space';
}
elsif($direct_par_def == 2) # for GUI
{
  read_set_file_for_GUI(\%param, $set_file, $set_file_default);
}
elsif($direct_par_def == 0)
{
  read_set_file_for_command_line(\%param, $set_file, \%param_list, $set_file_default);
}

check_param_limits(\%param, \%param_limits);

#determine the last created folder within $project_folder, that contains motif ('^data3');
my $pipe3_folder = determine_last_folder($param{project_folder}, '^pipe3');
my %pipe3_list = get_file_folder_list($pipe3_folder);
my $outfolder = time;
make_folder($param{syst}, $outfolder);
my $date = localtime $outfolder;

print "Input pipe3 folder: $pipe3_folder\n";
my $pipe4_folder = $param{project_folder}.'/pipe4_'.$outfolder;
make_folder($param{syst}, $pipe4_folder);
print "Output pipe4 folder: $pipe4_folder\n\n";
my $log_file = $pipe4_folder.'/pipe4_log_'.$outfolder.'.txt';


  unless (open(LOG, ">$log_file"))
  {print "Cannot open $log_file\n";}
  print LOG "**********************************************************\n";
  print LOG "QDD version2.1(beta) 28 June 2011\n";
  print LOG "Emese Meglecz, Aix-Marseille University, Marseille, France\n";
  print LOG 'emese.meglecz@univ-provence.fr';
  print LOG "\n";
  print LOG "Plesae, read the Documentation_QDD2.pdf\n";
  print LOG "**********************************************************\n\n";
  print LOG "Output folder identifyer: $outfolder\n";
  print LOG "Run started at : $date\n";
  print LOG "System (win/linux): $param{syst}\n";
  print LOG "Delete intremediate files (YES=1/NO=0): $param{del_files}\n";
  print LOG "Project folder: $param{project_folder}\n";
  print LOG "Output folder: $pipe4_folder\n";
  print LOG "\n************************************\n";


foreach my $file (sort keys %pipe3_list)
{

if ($file =~ /_pipe3_targets\.fas/)
{  
  my $time_beg = time;
  my $file_root = $file;
  $file_root =~ s/_targets\.fas//;  
  my $outfolder_spec = '';
  $outfolder_spec = $outfolder.'/'.$file_root.'_'.$outfolder;
  make_folder($param{syst}, $outfolder_spec);

  print "***********\n$file\n***********\n";
  print LOG "***********\n$file\n***********\n";
  my $blast_summary_file = $outfolder_spec.'/'.$file_root.'_summ_BLAST.csv';
  my $tax_file = $outfolder_spec.'/'.$file_root.'_tax.txt';

  my $remote_blast = 'perl remote_blast.pl "'.$pipe3_folder.'/'.$file.'" "'.$blast_summary_file.'" "'.$tax_file.'" 2>remote_blast_messages.txt';
  print LOG "$remote_blast\n";
  print "\nBLASTing sequences against GenBank\nThis step can be very long\n";
  system $remote_blast;
  
  my %taxonomy = ();
  read_taxonomy_data($tax_file, \%taxonomy); 
#  print Dumper(\%taxonomy);
  
  my $primer_file3 = $pipe3_folder.'/'.$file_root.'_primers.csv';
  my $primer_file4 = $pipe4_folder.'/'.$file_root.'_primers.csv';
  $primer_file4 =~ s/_pipe3_primers\.csv/_pipe4_primers\.csv/;
  pool_tax_blast_primer_data(\%taxonomy, $blast_summary_file, $primer_file3, $primer_file4);

  print LOG "\n\n";
  my $time_end = time;
  my $time_run = $time_end - $time_beg;
  print LOG "\nThe run took $time_run s.\n";
  
} # end if file motif
} # end foreach my $file

close LOG;

if ($param{del_files})
{
  delete_folder($outfolder, $param{syst});
}

print "\nThe anlysis is finished\n";
exit;


######################################################

sub define_param_number_pipe4
{
  my %param_list = (
      1=> ['syst', 'Operating system (win/linux):'], 
      2=> ['project_folder', 'Project folder:'], 
      3=> ['del_files', 'Delete intermediate files (YES=1/NO=0):']
      );
      
  del_files => '',

  return %param_list;
}


##################################################################

sub define_param_limits_pipe4
{
my ($param) = @_;
#hash of array [type(list, real, folder), error message, ['list of values/lower_upper limits)']
  my %param_limits = (
      'syst'=> ['list', 'Operating system must be win or linux', ['win', 'linux']], 
      'project_folder' => ['folder', "Cannot open the Project folder $$param{project_folder}"], 
      'del_files'=> ['list', 'del_files must be 0 or 1', [0,1]]
      );
  return %param_limits;
}
##################################################################

sub read_taxonomy_data
{
my ($tax_file, $taxonomy) = @_;
#my %taxonomy = (); # hash of hash $taxonomy{$TaxId}{species=> , superkingdom=> , kingdom=>, phylum=>, class=>, family=>, genus=>}

unless(open(TAX, $tax_file))
{
  print "Cannot open $tax_file\n";
}

my $id_data = '';
while(my $line = <TAX>)
{
  if ($line =~ /^<\/Taxon>$/)
  {
    $id_data =~ /(<TaxId>)(.+?)(<\/TaxId>)/m;
    my $taxId = $2;
    $id_data =~ /(<ScientificName>)(.+?)(<\/ScientificName>)/m;
    my $spec = $2;
    initialize_hash($spec, $taxId, $taxonomy);
    while ($id_data =~ /(<Taxon>)(.+?)(<\/Taxon>)/sg)
    {
      my $taxon = $2;
      $taxon =~ /(<Rank>)(.+?)(<\/Rank>)/m;
      {
        my $rank = $2;
        if(exists $$taxonomy{$taxId}{$rank})
        {
          $taxon =~ /(<ScientificName>)(.+?)(<\/ScientificName>)/m;
          $$taxonomy{$taxId}{$rank} = $2;
        }
      }
    }
    $id_data = '';
  }
  else
  {
    $id_data .= $line;
  }
}
close TAX;
}

#################################################
sub initialize_hash
{
my ($spec, $TaxId, $taxonomy) = @_;

$$taxonomy{$TaxId}{species} = $spec;
$$taxonomy{$TaxId}{superkingdom} ='';
$$taxonomy{$TaxId}{kingdom} ='';
$$taxonomy{$TaxId}{phylum} ='';
$$taxonomy{$TaxId}{class} ='';
$$taxonomy{$TaxId}{family} ='';
$$taxonomy{$TaxId}{genus} ='';
}

#################################################

sub pool_tax_blast_primer_data
{
my ($taxonomy, $blast_summary_file, $primer_file3, $primer_file4) = @_;

open_file_in($blast_summary_file, 'BL');
my %bl = ();
my $title_bl = <BL>;
chomp $title_bl;
while (my $line = <BL>)
{
  chomp $line;
  my @line = split(';', $line);
  $bl{$line[0]}[0] = $line[6];
  $bl{$line[0]}[1] = $line;
}
close BL;

open_file_in($primer_file3, 'PRIN');
open_file_out($primer_file4, 'OUT');
my $title = <PRIN>;
chomp $title;
print OUT "$title;$title_bl;superkingdom;kingdom;phylum;class;family;genus;species\n";
while (my $line = <PRIN>)
{
  chomp $line;
  my @line = split(';', $line);
  if (exists $bl{$line[0]})
  {
    print OUT "$line;$bl{$line[0]}[1];";
    print OUT "$$taxonomy{$bl{$line[0]}[0]}{superkingdom};$$taxonomy{$bl{$line[0]}[0]}{kingdom};$$taxonomy{$bl{$line[0]}[0]}{phylum};$$taxonomy{$bl{$line[0]}[0]}{class};$$taxonomy{$bl{$line[0]}[0]}{family};$$taxonomy{$bl{$line[0]}[0]}{genus};$$taxonomy{$bl{$line[0]}[0]}{species}\n";
  }
  else
  {
    print OUT "$line\n";
  }
}
close PRIN;
close OUT;
}
