#!/usr/bin/perl -w

# name of the program: sort_tag.pl
# sort sequences in a fasta file according to a list of tags

# reads all, sequences from a fasta file and identify which of the given tag is present, removes tag (and adaptor)
# writes one fasta file per tag with all sequences that had the given tag.
# if blast hit is shorter then adaptor, removes the adaptor length not the length of the hit

# INPUT: 	xxx.fas (fasta file with tags)
# 		yyy.fas fasta file to be cleaned and sorted out.

# OUPUT: yyy_m8.tbl results of blast againts te adapter file
#		yyy_tagY.fas 1 fasta file with all of the cleaned sequences that had the given tag
#		yyy_NOTAG.fas 1 fasta file with all of the cleaned sequences that did not have tag


  use warnings;
  use strict;
  use subprogramQDD;

my $syst = '';
my $fas_file = ''; # file wo path
my $fas_folder = ''; # path to fas_file
my $outfolder = '';
my $Evector = '';
my $word_size = '';
my $blast_path = '';
my $blast_version = '';
my $tag_file = ''; # file wo path
#my $tag_folder = ''; # path to tag_file

  	
if ( scalar@ARGV == 9 )
{
   	$syst = $ARGV[0];
   	$fas_file = $ARGV[1];
    $fas_folder= $ARGV[2];
   	$outfolder = $ARGV[3];
   	$blast_path = $ARGV[4];
   	$blast_version = $ARGV[5];
   	$Evector = $ARGV[6];
   	$word_size = $ARGV[7];
   	$tag_file = $ARGV[8];
#    $tag_folder = $ARGV[9];
}
else
{
	$syst = 'win';
	$fas_file = 'Ea tag.fas';
	$fas_folder= 'd:/test with space';
	$outfolder = 'temp1';
	$Evector = '1E-1';
	$word_size = 7;
#	$blast_path = 'c:/BLAST2_2_18/bin/';
	$blast_path = 'c:/Program files/NCBI/blast-2.2.25+/bin/';
	$blast_version = 'blast+';
  $tag_file = 'tag.fas'; # if blast+ is used tag.fas must be in the same folder as sort_tag.pl
#  $tag_folder = 'd:/pipe_clean/QDD2';
  print "Direct parameters are used in sort_tag.pl\n";
}


my %adapt = ();
my %seq = ();
my %sort = ();

  my $fas_file_root = $fas_file;
  $fas_file_root =~ s/\..+$//;
	my $outfile = $outfolder.'/'.$fas_file_root.'_NOTAG.tag';
	my $out_blast = $outfolder.'/'.$fas_file_root.'_m8.tbl';

  $fas_file = $fas_folder.'/'.$fas_file;
#  $tag_file = $tag_folder.'/'.$tag_file;
  
# blast sequence file againts the adaptor file; filter is switched off, *
#to be able to clean even low complexity sequences
call_blast_adapter_no_filter_word_size($fas_file, $out_blast, $tag_file, $Evector, $blast_path, $blast_version, $word_size, $syst);

%seq = read_hach_from_fasta_code_wo_space($fas_file);
 my %seq_l = length_sequences(\%seq);
%adapt = read_hach_from_fasta_code_wo_space($tag_file);
my %adapt_l = length_sequences(\%adapt);

open_file_in($out_blast, 'BL');
while(my $line = <BL>)
{
	my @line = split(' ', $line);
# hit sur brin +, hit at the begining of the sequence (+2 for to allow for 1 seq error (indel))
	if ($line[8] < $line[9] and $line[7] < $adapt_l{$line[1]} +2)
	{
		$sort{$line[1]} .= $line[0].';';
		if ($line[7]> $adapt_l{$line[1]})
		{
			$seq{$line[0]} = substr($seq{$line[0]}, $line[7], $seq_l{$line[0]} - $line[7]);
		}
		else
		{
			$seq{$line[0]} = substr($seq{$line[0]}, $adapt_l{$line[1]}, $seq_l{$line[0]} - $adapt_l{$line[1]});	
		}
	}
}

close (BL);

#print_hachage(\%sort);

foreach my $adapt (keys %sort)
{
	my @list = split(';', $sort{$adapt});
	my $out = $outfolder.'/'.$fas_file_root.'_'.$adapt.'.tag';
	open_file_out($out, 'OUT');
	my $i = 0;
	foreach my $code (@list)
	{
		if ($seq{$code} eq '')
		{
			print "In $code an adapter is already removed\n";
		}
		else
		{
			print OUT ">$code";
			print OUT "_$adapt\n$seq{$code}\n";
			$seq{$code} = '';
			++$i;
		}
	}
	close OUT;
	print "Number of sequences with $adapt tag: $i\n";
}
open_file_out($outfile, 'OUT');

# prints a file with the non_cleaned sequences
my $i = 0;
foreach my $code (keys %seq)
{
	unless ($seq{$code} eq '')
	{
		print OUT ">$code\n$seq{$code}\n";
		++$i;
	}
}
	print "Number of sequences without tag: $i\n";

close (OUT);

exit;

